<?php

/**
 * crossSiteAction class
 */
class crossSiteAction
{

    public function setData($dataArr = array())
    {
        // get database
        $db = Database::getDatabase(true);

        // get auth
        $Auth = Auth::getAuth();

        // generate unique keys and check for existing
        $exists  = true;
        $tracker = 0;
        while (($exists !== false) && ($tracker < 10))
        {
            // unique keys
            $key1 = hash('sha256', microtime() . rand(100000, 999999));
            $key2 = hash('sha256', microtime() . rand(100000, 999999) . $key1);

            $exists = self::getData($key1, $key2);
            $tracker++;
        }

        // failed too many times so ext
        if ($tracker >= 10)
        {
            return false;
        }

        // get user id
        $userId = '';
        if ($Auth->loggedIn())
        {
            $userId = $Auth->id;
        }

        // add user id to overall data
        $dataArr['_logged_in_user_id'] = $userId;

        // insert into database
		$setData = $db->query('INSERT INTO cross_site_action (key1, key2, data, date_added) VALUES (' . $db->quote($key1) . ',  ' . $db->quote($key2) . ', '.$db->quote(json_encode($dataArr)).', NOW())');

        return array('key1' => $key1, 'key2' => $key2);
    }

    public function getData($key1, $key2)
    {
        // get database
        $db = Database::getDatabase(true);

        // load data from database
        $getData = $db->getValue('SELECT data FROM cross_site_action WHERE key1=' . $db->quote($key1) . ' AND key2=' . $db->quote($key2) . ' LIMIT 1');
        if (($getData !== false) && strlen($getData))
        {
            return json_decode($getData, true);
        }

        return false;
    }
    
    public function deleteData($key1, $key2)
    {
        // get database
        $db = Database::getDatabase(true);

        // delete data from database
        $db->query('DELETE FROM cross_site_action WHERE key1=' . $db->quote($key1) . ' AND key2=' . $db->quote($key2) . ' LIMIT 1');
    }
    
    public function purgeOldData()
    {
        // get database
        $db = Database::getDatabase(true);
        
        // delete anything older than 10 minutes
        $db->query('DELETE FROM cross_site_action WHERE date_added < DATE_SUB(NOW(), INTERVAL 10 MINUTE)');
    }

    public function setAuthFromKeys($key1 = '', $key2 = '', $deleteCrossSiteAction = true)
    {
        // make sure we have some keys
        if((strlen($key1) == 0) && (strlen($key2) == 0))
        {
            return false;
        }
        
        // get auth
        $Auth = Auth::getAuth();

        // check for existing Auth, disabled, not needed as covered previously
        //if ($Auth->loggedIn() == true)
        //{
        //    return $Auth;
        //}
        
        // check referer
        $refDomain = strtolower(coreFunctions::getReffererDomainOnly());
        if (!$refDomain)
        {
            return false;
        }
        else
        {
            // make sure host is what we expect
            if($refDomain != strtolower(_CONFIG_CORE_SITE_HOST_URL))
            {
                return false;
            }
        }

        // try to load the data
        $dataArr = self::getData($key1, $key2);
        if (!$dataArr)
        {
            return false;
        }
        
        // remove database data
        if($deleteCrossSiteAction == true)
        {
            self::deleteData($key1, $key2);
        }
        
        // remove any old database data
        self::purgeOldData();

        // try to load the Auth object
        if (!isset($dataArr['_logged_in_user_id']))
        {
            return false;
        }

        return $Auth->impersonate((int) $dataArr['_logged_in_user_id']);
    }
    
    public static function appendUrl($url, $dataArr = array())
    {
        // create tracker key for session
        $sessionKeys = self::setData($dataArr);
        if($sessionKeys == false)
        {
            return $url;
        }

        // prepare url
        if(strpos($url, '?') !== false)
        {
            $url .= '&';
        }
        else
        {
            $url .= '?';
        }
        
        // add keys to url
        return $url . 'csaKey1=' . $sessionKeys['key1'] . '&csaKey2=' . $sessionKeys['key2'];
    }

}
